#!/bin/bash

export PATH=$PATH:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin

hw_info=$(cat /etc/hw_version 2> /dev/null | sed -e 's/[ ][ ]*/_/g')
current_arch=$(uname -m)
os=""
path_to_usbip_ko=""
root_password=""
current_path=""
gnusb_dir=""

function Usage
{
cat << END
******************************************************************************
* This script is used to build usbip-core.ko and usbip-host.ko
******************************************************************************

END
	exit
}

function clean_usbip_src
{
	if [[ -z $gnusb_dir ]]; then
		exit
	fi
	rm -rf ${gnusb_dir}driver_auto_build/__wasd_uos_kernel_usbip/
	rm -rf ${gnusb_dir}driver_auto_build/__wasd_kylin_kernel_usbip/
}

function get_gnusb_dir
{
	if [ "$os" == 'kylin' ]; then
		gnusb_dir=$(find /usr/local -name "libgnusbiphelper.so" 2> /dev/null | grep -E "^/usr/local" | sed 's/libgnusbiphelper.so//g')
	elif [ "$os" == 'uos' ]; then
		gnusb_dir=$(find /opt/apps -name "libgnusbiphelper.so" 2> /dev/null | grep -E "^/opt/apps" | sed 's/libgnusbiphelper.so//g')
	else
		exit
	fi
}

function gnlog
{
	cur_date=$(date)
	log_fmt=$*
	echo "${cur_date}:${log_fmt}"
}

function get_os_ver_info_kylin
{
	kylin_build=$(cat /etc/kylin-build 2> /dev/null | grep ^Build | awk '{print $2}')
	if [[ -z $hw_info ]]; then
		gnlog "此系统CPU不是华为麒麟990"
		path_to_usbip_ko="kylin_${current_arch}_${kylin_build}"
	else
		gnlog "此系统CPU是华为麒麟990 代号为${hw_info}"
		path_to_usbip_ko="kylin_kirin990_${kylin_build}_${hw_info}"
	fi
	gnlog "银河麒麟 usb驱动打包信息检测完成"
}

function get_os_ver_info_uos
{
	uos_ver=$(cat /etc/os-version 2> /dev/null | grep ^MinorVersion= | awk -F= '{print $2}')
	if [[ -z $hw_info ]]; then
		gnlog "此系统CPU不是华为麒麟990"
		path_to_usbip_ko="uos_${current_arch}_${uos_ver}"
	else
		gnlog "此系统CPU是华为麒麟990 代号为${hw_info}"
		path_to_usbip_ko="uos_kirin990_${uos_ver}_${hw_info}"
	fi	
	gnlog "UOS系统 usb驱动打包信息检测完成"
}

function build_and_zip_ko
{
	ko_path_kylin="__wasd_kylin_kernel_usbip/usbip/"
	ko_path_uos="__wasd_uos_kernel_usbip/usbip/"
	ko_path="NULL"
	if [ "$os" == 'kylin' ]; then
		ko_path="${ko_path_kylin}"
	elif [ "$os" == 'uos' ]; then
		ko_path="${ko_path_uos}"
	else
		gnlog "must not reach here ! $OS"
		exit
	fi
	
	build_cmd="cd ${current_path}/${ko_path} && make && mv *.ko ../../ && cd ../../ && chmod 755 *.ko && zip ${path_to_usbip_ko}.zip *.ko"
	
	if [[ -z $root_password ]]; then
		cd ${current_path}/${ko_path} && make && mv *.ko ../../ && cd ../../ && chmod 755 *.ko
	else
		echo "${root_password}" | sudo -S bash -c "${build_cmd}"
		echo "${root_password}" | sudo -S bash -c "cd ${current_path}/ && ls ./${path_to_usbip_ko}.zip && chmod 755 ./${path_to_usbip_ko}.zip && echo "打包成功，文件名为${path_to_usbip_ko}.zip" || echo "编译打包失败！""
	fi
}

function insert_to_appcloud
{
	echo "正在安装usbip内核驱动，请稍后。。。。"
	if [[ -z $gnusb_dir ]]; then
		get_gnusb_dir
	fi
	gnlog "gnusb路径为 ${gnusb_dir}"
	if [[ -z $root_password ]]; then
		cd ${current_path}/ && ls *.ko && cp -f *.ko ${gnusb_dir}
	else
		echo "${root_password}" | sudo -S bash -c "cd ${current_path}/ && ls *.ko && cp -f *.ko ${gnusb_dir}"
	fi
}

function start_build
{
	if [ "$os" == 'kylin' ]; then
		gnlog "检测到正在运行银河麒麟系统"
		get_os_ver_info_kylin
	elif [ "$os" == 'uos' ]; then
		gnlog "检测到正在运行UOS系统"
		get_os_ver_info_uos
	else
		gnlog "usb驱动自动编译打包功能暂不支持当前系统$OS"
		exit
	fi
	
	build_and_zip_ko
	insert_to_appcloud
	gnlog "运行完成"
}

function running_by_double_click
{
	gnlog "当前路径为 ${current_path}"
	current_path=$(pwd)
	read -s -p "请输入root用户密码或开机登录密码，输入后按下enter键:" root_password
	echo -e "\n"
	start_build $*
	sleep 10s
	exit
}

trim() {
    str=""
    if [ $# -gt 0 ]; then
        str="$1"
    fi
    echo "$str" | sed -e 's/^[ \t\r\n]*//g' | sed -e 's/[ \t\r\n]*$//g'
}
os=$(trim $(cat /etc/os-release 2>/dev/null | grep ^ID= | awk -F= '{print $2}'))
if [ "$os" = "" ]; then
    os=$(trim $(lsb_release -i 2>/dev/null | awk -F: '{print $2}'))
fi
if [ ! "$os" = "" ]; then
    os=$(echo $os | tr '[A-Z]' '[a-z]')
fi
echo "os=${os}"

function main
{
	[ "$#" -lt 1 ] && running_by_double_click
	case "$1" in
		asroot)
			[ "$#" -lt 2 ] && Usage
			current_path="$2"
			start_build $*
			clean_usbip_src
			;;
		*)
			Usage
			;;
	esac
	return
}

main $*
