#!/bin/bash

export PATH=$PATH:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin

hw_info=$(cat /etc/hw_version 2> /dev/null | sed -e 's/[ ][ ]*/_/g')
current_arch=$(uname -m)
path_to_usbip_ko="NULL"
disk_path_to_usbip_ko="NULL"
os="NULL"
is_exist_usbip_ko="NULL"
pack_name=$*

function gnlog
{
	cur_date=$(date)
	log_fmt=$*
	echo "${cur_date}:${log_fmt}"
}

function wait_until_dpkg_return
{
	while true
	do
		ps_ret=$(ps aux 2> /dev/null | grep "[d]pkg -i" | awk '{print (length($0)>10) ? "gnusb_yes" : ""}')
		if [ "$ps_ret" == 'gnusb_yes' ]; then
			gnlog "dpkg is running"
			sleep 1s
		else
			gnlog "dpkg has been exited!"
			break
		fi
	done
}

function install_usbip_utils
{
	cd_usbip_install_path=$(cd /opt/apps/${pack_name}/files/gnusb/usbip_utils/`uname -m` 2> /dev/null && echo gnusb_yes)
	if [ "$cd_usbip_install_path" == 'gnusb_yes' ]; then
		gnlog "kylin 找到${current_arch}的usbip-util安装包，开始安装"
		if [ -e "/opt/apps/${pack_name}/files/gnusb/usbip_utils/${current_arch}/usbipd" ]; then
			gnlog "kylin 安装自编译usbip_utils"
			ln -s /opt/apps/${pack_name}/files/gnusb/usbip_utils/`uname -m`/usbip /bin/usbip && ln -s /opt/apps/${pack_name}/files/gnusb/usbip_utils/`uname -m`/usbipd /bin/usbipd
			return
		fi
		wait_until_dpkg_return
		dpkg -i /opt/apps/${pack_name}/files/gnusb/usbip_utils/`uname -m`/*.deb
	else
		gnlog "DEB未找到当前架构${current_arch}的usbip-util安装包！usb映射功能可能工作不正常"
	fi
}

function install_usbip_utils_uos
{
	cd_usbip_install_path=$(cd /opt/apps/${pack_name}/files/gnusb/usbip_utils/`uname -m` 2> /dev/null && echo gnusb_yes)
	if [ "$cd_usbip_install_path" == 'gnusb_yes' ]; then
		gnlog "UOS 1找到当前架构${current_arch}的usbip-util安装包，开始安装"
		if [ -e "/opt/apps/${pack_name}/files/gnusb/usbip_utils/${current_arch}/usbipd" ]; then
			gnlog "UOS 安装自编译usbip_utils"
			ln -s /opt/apps/${pack_name}/files/gnusb/usbip_utils/`uname -m`/usbip /bin/usbip && ln -s /opt/apps/${pack_name}/files/gnusb/usbip_utils/`uname -m`/usbipd /bin/usbipd
			return
		fi
		wait_until_dpkg_return
		nohup dpkg -i /opt/apps/${pack_name}/files/gnusb/usbip_utils/`uname -m`/*.deb &
		gnlog "UOS 1 usbip install completed"
	else
		gnlog "UOS 1 未找到当前架构${current_arch}的usbip-util安装包！usb映射功能可能工作不正常"
	fi
}

function check_usbip_utils
{
	usbip_util_check_cmd=$(usbip version 2> /dev/null | grep "usbip-utils" | awk '{print (length($0)>10) ? "gnusb_yes" : ""}')
	if [ "$usbip_util_check_cmd" == 'gnusb_yes' ]; then
		gnlog "系统中已存在usbip用户模式程序"
	else
		gnlog "系统中不存在usbip用户模式程序，准备按架构进行安装"
		
		deb_pack_name=$pack_name
		if [[ -z $deb_pack_name ]]; then
			gnlog "没有收到包名，选择银河麒麟的安装路径"
			install_usbip_utils
		else
			gnlog "收到包名${deb_pack_name}，选择UOS的安装路径"
			install_usbip_utils_uos $deb_pack_name
		fi
	fi
}

function main
{
	current_time=$(date)
	gnlog "${current_time}:开始检测本地usb映射环境"
	check_usbip_utils
	gnlog "${current_time}:usb映射环境检测完毕"
}

trim() {
    str=""
    if [ $# -gt 0 ]; then
        str="$1"
    fi
    echo "$str" | sed -e 's/^[ \t\r\n]*//g' | sed -e 's/[ \t\r\n]*$//g'
}
os=$(trim $(cat /etc/os-release 2>/dev/null | grep ^ID= | awk -F= '{print $2}'))
if [ "$os" = "" ]; then
    os=$(trim $(lsb_release -i 2>/dev/null | awk -F: '{print $2}'))
fi
if [ ! "$os" = "" ]; then
    os=$(echo $os | tr '[A-Z]' '[a-z]')
fi
echo "os=${os}"

main $*
